import unittest
from pucktrick.duplicated import *
from pucktrick.utils import *
import pandas as pd
import numpy as np
from pandas.testing import assert_frame_equal


class Test_outlies(unittest.TestCase):
 
 def test_duplicateClassExtended(self):
      percentage=0.5
      num_rows = 1000
      fake_df=create_fake_table(num_rows)
      target='f3'
      value='banana'
      old_len=len(fake_df[fake_df[target]==value])
      new_percentage=0.7
      config=create_json_for_feature( new_percentage, "new")
      strategy = config.get("strategy", {})
     
      noise_df=duplicate(fake_df,strategy)
      new_len=len(noise_df[noise_df[target]==value])
      diff=round((new_len-old_len)/old_len,1)
      self.assertEqual(diff, new_percentage)
  
 def test_duplicateAllExtended(self):
      percentage=0.5
      num_rows = 1000
      fake_df=create_fake_table(num_rows)
      config=create_json_for_feature( percentage, "new")
      strategy = config.get("strategy", {})
      noise_df=duplicate(fake_df,strategy)
      new_percentage=0.7
      config=create_json_for_feature( new_percentage, "extended")
      strategy = config.get("strategy", {})
      
      noise_df=duplicate(noise_df,strategy,fake_df)
      new_len=len(noise_df)
      diff=(new_len-num_rows)/num_rows
      self.assertEqual(diff, new_percentage)

if __name__ == "__main__":
    unittest.main()



import unittest
from pucktrick.noisy import *
from pucktrick.utils import * 
import pandas as pd
from pandas.testing import assert_frame_equal
class TestCategorical(unittest.TestCase):
    def test_saluta(self):
        percentage=0.5
        num_rows = 1000
        fake_df=create_fake_table()
        column='f3'
        strategy =  {'affected_features': 'f3', 'selection_criteria': 'all', 'percentage': 0.5, 'mode': 'new', 'perturbate_data': {'distribution': 'random', 'param': None, 'value': None, 'condition_logic': None}}
        noise_df=noise(fake_df,strategy)
        dif = fake_df[column] != noise_df[column]
        diff_number = dif.sum()
        percentage=0.7
        strategy =  {'affected_features': 'f3', 'selection_criteria': 'all', 'percentage': 0.7, 'mode': 'extended', 'perturbate_data': {'distribution': 'random', 'param': None, 'value': None, 'condition_logic': None}}
        noise_df=noise(noise_df,strategy,fake_df)
        dif = fake_df[column] != noise_df[column]
        diff_number = dif.sum()
        diff_p=diff_number/num_rows
    
        # Verifica che il numero di modifiche sia corretto
        self.assertEqual(diff_p, percentage)
    


if __name__ == "__main__":
    unittest.main()



def labels(train_df, original_df=None, strategy=None):

    with open(strategy, 'r') as file:
        strategy = json.load(file)

    strategy = strategy.get("strategy", {})
    target = strategy.get("affected_features")
    selection_criteria = strategy.get("selection_criteria")
    percentage = strategy.get("percentage")
    mode = strategy.get("mode")
    perturbate_config = strategy.get("perturbate_data", {})
    distr1 = perturbate_config.get("distribution")
    value = perturbate_config.get("value", [])
    condition_logic = perturbate_config.get("condition_logic", None)
    params= perturbate_config.get("param", {})
    number=None

    target_col = target[0] if isinstance(target, list) else target
    unique_values = train_df[target_col].dropna().unique()  
    if len(unique_values) == 2:
        var_type = "binary"
    elif len(unique_values) > 2:
        var_type = "categorical"
    else:
        raise ValueError(f"Unable to determine the type of target column '{target}'.")


    if var_type == "binary":
        #print("Target is binary")
        if mode == 'extended':
            noise_df= noiseBinaryExt(original_df, train_df, target_col,distr1, percentage,number,params)
        else:
            noise_df = noiseBinaryNew(train_df, target_col, distr1, percentage,params,number) 

    elif var_type == "categorical":
        if mode == 'extended':
            noise_df = noiseCategoricalIntExtendedExistingValues(original_df, train_df, target_col,distr1, percentage,number,params)

        else:
            noise_df = noiseCategoricalIntNewExistingValues(train_df, target_col, distr1, percentage,number,params) 
    for col in noise_df.select_dtypes(include=['int64']).columns:
        noise_df[col] = noise_df[col].astype('Int64')  

    return noise_df



import unittest
from pucktrick.outliers import *
from pucktrick.utils import *
import pandas as pd
import numpy as np
from pandas.testing import assert_frame_equal

class Test_outlies(unittest.TestCase):
  
  def test_outliers(self):
        percentage=0.5
        num_rows = 1000
        fake_df=create_fake_table(num_rows)
        column='date'
        strategy =  {'affected_features': column, 'selection_criteria': 'all', 'percentage': percentage, 'mode': 'new', 'perturbate_data': {'distribution': 'random', 'param': None, 'value': None, 'condition_logic': None}}
        noise_df=outlier(fake_df,strategy)
        dif = fake_df[column] != noise_df[column]
        diff_number = dif.sum()
        percentage=0.7
        strategy =  {'affected_features': column, 'selection_criteria': 'all', 'percentage':percentage, 'mode': 'extended', 'perturbate_data': {'distribution': 'random', 'param': None, 'value': None, 'condition_logic': None}}
        noise_df=outlier(noise_df,strategy,fake_df)
        dif = fake_df[column] != noise_df[column]
        diff_number = dif.sum()
        diff_p=diff_number/num_rows
        self.assertEqual(diff_p, percentage)
  
    


if __name__ == "__main__":
    unittest.main()
